package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.RandomAccessFile;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.event.*;
import java.util.Vector;
import objects.*;
import util.*;

public class CGMainFrm extends JFrame implements ActionListener, KeyListener
{
	private static final long serialVersionUID = 000000l;
	
	// GUI Elements
	private static JMenuBar mbar;
	private static JPanel contentPane, objectPane, mapInfoPane, bufInfoPane, objInfoPane;
	private static CGPaintComponent viewComponent;
	private static JScrollPane pictureScrollPane, tableScrollPane;
	private static JTable table;
	private static JButton addButton;
	private static JLabel bufLabel, mapLabel, objLabel;
	private static String mapString, bufString, objString;
	private static String mapPath, bufPath, objPath;
	private static JProgressBar pb;
	private static CGSettingFrm setFrame;
	private static CGHeaderFooterFrm hefoFrame;
	
	// Other Elements
	private static CGObjects obj;
	private static CGFloatBuffer buf;
	
	public CGMainFrm(){
		super( "CG Map Editor" );
		initialize();
	}
	
	private void initialize(){
		obj = new CGObjects();
		bufString = "Heightbuffer: No Buffer loaded!";
		if(bufPath == null) bufPath = "";
		mapString = "Map: No Map loaded!";
		if(mapPath == null) mapPath = "";
		objString = "Objects Table: No Objects loaded!";
		if(objPath == null) objPath = "";
		
		if (setFrame == null) setFrame = new CGSettingFrm();
		if (hefoFrame == null) hefoFrame = new CGHeaderFooterFrm();
		
		this.addKeyListener(this);
		this.setJMenuBar(getMenuBar1());
		this.setContentPane(getMyContentPane());

		this.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		this.setBounds( 10, 10, 1260, 800 );
	}
	
	private JPanel getMyContentPane() {
		if (contentPane == null) {
			contentPane = new JPanel();
			contentPane.setOpaque(true);
			contentPane.setLayout(new BorderLayout());
			viewComponent = new CGPaintComponent();
			viewComponent.setPreferredSize(viewComponent.getPreferredSize());
			pictureScrollPane = new JScrollPane();
			pictureScrollPane.createVerticalScrollBar();
			pictureScrollPane.createHorizontalScrollBar();
			pictureScrollPane.setPreferredSize(new Dimension(600, 600));
			pictureScrollPane.setViewportBorder(BorderFactory.createLineBorder(Color.black));
			pictureScrollPane.setVisible(true);
			pictureScrollPane.setWheelScrollingEnabled(true);
			contentPane.add(pictureScrollPane, BorderLayout.CENTER);
			contentPane.add(getObjectPane(), BorderLayout.WEST);
		}
		return contentPane;
	}
	
	private JMenuBar getMenuBar1(){
		if (mbar == null){
			mbar = new JMenuBar();
			mbar.addKeyListener(this);
			// File menu
			JMenu menu = new JMenu("File");
			// Load the map image
			JMenuItem item = new JMenuItem("Load Map Image");
			item.setActionCommand("map");
			item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, KeyEvent.CTRL_MASK));
			item.addActionListener(this);
			menu.add(item);
			// Load the height buffer
			JMenuItem item2 = new JMenuItem("Load Height Buffer");
			item2.setActionCommand("buf");
			item2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, KeyEvent.CTRL_MASK));
			item2.addActionListener(this);
			menu.add(item2);
			// Load the objects file
			JMenuItem item3 = new JMenuItem("Load Objects Table");
			item3.setActionCommand("obj");
			item3.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_J, KeyEvent.CTRL_MASK));
			item3.addActionListener(this);
			menu.add(item3);
			menu.addSeparator();
			// Save the current project
			JMenuItem item4 = new JMenuItem("Save designed Level");
			item4.setActionCommand("save");
			item4.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_MASK));
			item4.addActionListener(this);
			menu.add(item4);
			// Load a project
			JMenuItem item5 = new JMenuItem("Open designed Level");
			item5.setActionCommand("load");
			item5.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));
			item5.addActionListener(this);
			menu.add(item5);
			menu.addSeparator();
			// Save the current project
			JMenuItem item6 = new JMenuItem("Convert to XML File");
			item6.setActionCommand("create");
			item6.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_MASK));
			item6.addActionListener(this);
			menu.add(item6);
			mbar.add(menu);
			// Setting menu
			JMenu menu2 = new JMenu("Settings");
			// Set the multiplication factor for [0;1] normed positions and sizes
			JMenuItem item7 = new JMenuItem("Multiplication Factor");
			item7.setActionCommand("multi");
			item7.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, KeyEvent.CTRL_MASK));
			item7.addActionListener(this);
			menu2.add(item7);
			// Set header and footer
			JMenuItem item8 = new JMenuItem("Header and Footer");
			item8.setActionCommand("hefo");
			item8.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, KeyEvent.CTRL_MASK));
			item8.addActionListener(this);
			menu2.add(item8);
			mbar.add(menu2);
		}
		return mbar;
	}
	
	private JPanel getObjectPane(){
		if (objectPane == null){
			objectPane = new JPanel();
			objectPane.addKeyListener(this);
			objectPane.setLayout(new BoxLayout(objectPane,BoxLayout.Y_AXIS));
			objectPane.add(getObjectScrollPane());
			objectPane.add(getAddButton());
			objectPane.add(getMapInfoPane());
			objectPane.add(getObjInfoPane());
			objectPane.add(getBufInfoPane());
			objectPane.add(new JLabel(" "));
			// Add a progress bar to show progress of heightbuffer loading
			pb = new JProgressBar(0, 1024*1024*4);
			pb.setValue(0);
			pb.setBorderPainted(false);
			pb.setStringPainted(true);
			pb.setVisible(false);
			objectPane.add(pb);
		}
		return objectPane;
	}
	
	private JScrollPane getObjectScrollPane(){
		if (tableScrollPane == null){
			tableScrollPane = new JScrollPane();
			tableScrollPane.addKeyListener(this);
			tableScrollPane.createVerticalScrollBar();
			tableScrollPane.createHorizontalScrollBar();
			tableScrollPane.setPreferredSize(new Dimension(200, 800));
			tableScrollPane.setViewportBorder(BorderFactory.createLineBorder(Color.black));
			tableScrollPane.setVisible(true);
			tableScrollPane.setWheelScrollingEnabled(true);
			tableScrollPane.setViewportView(getObjectTable());
		}
		return tableScrollPane;
	}
	
	private JTable getObjectTable(){
		if (table == null){
			table = new JTable(obj.getData(), obj.getColumnNames());
			table.addKeyListener(this);
			table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			ListSelectionModel rowSM = table.getSelectionModel();
			rowSM.addListSelectionListener(new ListSelectionListener() {
			    public void valueChanged(ListSelectionEvent e) {
			        //Ignore extra messages.
			        if (e.getValueIsAdjusting()) return;
			        ListSelectionModel lsm = (ListSelectionModel)e.getSource();
			        if (lsm.isSelectionEmpty()) {
			            //no rows are selected
			        }
			        else {
			            int selectedRow = lsm.getMinSelectionIndex();
			            obj.setSelectedIndex(selectedRow);
			        }
			    }
			});
		}
		return table;
	}

	private JButton getAddButton(){
		if (addButton == null){
			addButton = new JButton(">> Add Object >>");
			addButton.addKeyListener(this);
			addButton.setAlignmentX(Component.CENTER_ALIGNMENT);
			addButton.setActionCommand("addobject");
			addButton.addActionListener(this);
		}
		return addButton;
	}
	
	private JPanel getMapInfoPane() {
		if (mapInfoPane == null) {
			mapInfoPane = new JPanel();
			mapInfoPane.addKeyListener(this);
			mapInfoPane.add(getMapLabel());
			mapInfoPane.setSize(new Dimension(100,20));
			mapInfoPane.setPreferredSize(new Dimension(100,20));
		}
		return mapInfoPane;
	}
	
	private JLabel getMapLabel(){
		if (mapLabel == null){
			mapLabel = new JLabel(mapString);
			mapLabel.addKeyListener(this);
			mapLabel.setForeground(Color.RED);
		}
		return mapLabel;
	}
	
	private JPanel getBufInfoPane() {
		if (bufInfoPane == null) {
			bufInfoPane = new JPanel();
			bufInfoPane.addKeyListener(this);
			bufInfoPane.add(getBufLabel());
			bufInfoPane.setSize(new Dimension(100,20));
			bufInfoPane.setPreferredSize(new Dimension(100,20));
		}
		return bufInfoPane;
	}
	
	private JLabel getBufLabel(){
		if (bufLabel == null){
			bufLabel = new JLabel(bufString);
			bufLabel.addKeyListener(this);
			bufLabel.setForeground(Color.RED);
		}
		return bufLabel;
	}
	
	private JPanel getObjInfoPane() {
		if (objInfoPane == null) {
			objInfoPane = new JPanel();
			objInfoPane.addKeyListener(this);
			objInfoPane.add(getObjLabel());
			objInfoPane.setSize(new Dimension(100,20));
			objInfoPane.setPreferredSize(new Dimension(100,20));
		}
		return objInfoPane;
	}
	
	private JLabel getObjLabel(){
		if (objLabel == null){
			objLabel = new JLabel(objString);
			objLabel.addKeyListener(this);
			objLabel.setForeground(Color.RED);
		}
		return objLabel;
	}
	
	// This method returns the objects object
	public static CGObjects getObjects(){
		return obj;
	}
	
	// This method returns the buffer
	public static CGFloatBuffer getBuffer(){
		return buf;
	}
	
	// This method returns the progress bar
	public static JProgressBar getProgressBar(){
		return pb;
	}
	
	// This method returns the added objects in a vector
	public static Vector getAddedObjects(){
		return viewComponent.getObjectVector();
	}
	
	// This method returns the added objects in a vector
	public static void addObjectDirect(CGAddedObject ao){
		viewComponent.addObject(ao);
	}
	
	// This method returns the multiplication factor for sizes
	public static float getFactor(){
		return setFrame.getFactor();
	}
	
	// This method returns the multiplication factor for sizes
	public static void setFactor(float f){
		setFrame.setFactor(f);
	}
	
	// This method returns the header of the xml file
	public static String getHeader(){
		return hefoFrame.getHeader();
	}
	
	// This method returns the footer of the xml file
	public static String getFooter(){
		return hefoFrame.getFooter();
	}
	
	// This method returns the paths to the map, object and buffer file
	public static String[] getPaths(){
		String[] s = new String[]{mapPath,bufPath,objPath};
		// System.out.println("1: " + s[0]);
		// System.out.println("2: " + s[1]);
		// System.out.println("3: " + s[2]);
		return s;
	}
	
	// Sets an Image directly
	public static void setImageFromPath(String p){
		File file = new File (p);
		if (file != null) {
			mapPath = file.getPath();
			mapString = "Map: " + file.getName();
			mapLabel.setForeground(new Color(50,150,50));
			mapLabel.setText(mapString);
			viewComponent.setImage( file );
			viewComponent.setDoubleBuffered(true);
			viewComponent.addMouseListener(viewComponent);
			// viewComponent.addKeyListener(this);
		}
		pictureScrollPane.setViewportView(viewComponent);
	}
	
	// Method to set the Buffer directly
	public static void setBufferFromPath(String p){
		final File file = new File(p);
		if (file != null) {
			// TODO: Load Buffer
			bufPath = file.getPath();
			bufString = "Loading Heightbuffer...";
			bufLabel.setForeground(new Color(50,50,150));
			bufLabel.setText(bufString);
			try {
				final RandomAccessFile raf = new RandomAccessFile(file,"r");
				if (buf == null) buf = new CGFloatBuffer(1024,1024);
				// THREAD
				new Thread() {
					public void run() {
						pb.setVisible(true);
						buf.fillBuffer(raf);
						bufString = "Heightbuffer: " + file.getName();
						bufLabel.setForeground(new Color(50,150,50));
						bufLabel.setText(bufString);
					}
				}.start();
			}
			catch (Exception E){
				System.out.println("File not found: " + file.getName());
			}		
		}
	}

	// Method to set the Objects table directly
	public static void setObjectsFromPath(String p){
		File file = new File(p);
		if (file != null) {
			objPath = file.getPath();
			try {
				// TODO: Load Table of Objects	
				RandomAccessFile raf = new RandomAccessFile(file,"r");
				CGLoadFileParser.parseObjects(raf);
				raf.close();
				table = null;
				if (obj == null) obj = new CGObjects();
				objString = "Objects Table: " + file.getName();
				objLabel.setForeground(new Color(50,150,50));
				objLabel.setText(objString);
			}
			catch (Exception E){
				System.out.println("File not found: " + file.getName());
				System.out.println("Error: " + E.getMessage());
			}					
		}
	}
	
	// Get Info if all Objects are loaded
	private boolean everythingSet(){
		if(viewComponent!=null && buf!=null && obj!=null && !pb.isVisible()){
			return true;
		}
		else{
			// TODO: Nachrichtenfenster
			JOptionPane.showMessageDialog(this,
				    "Error: Map Image, Height Buffer or Objects Table has not been loaded.\n" +
				    "The XML file was not built.",
				    "Error on XML file generation",
				    JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}
	
	public void actionPerformed( ActionEvent e ) {
		// Load the map picture
		if ("map".equals(e.getActionCommand())){
			JFileChooser d = new JFileChooser();
			d.setFileFilter( new FileFilter() {
				public boolean accept(File f)
				{
					return f.isDirectory() || f.getName().toLowerCase().endsWith(".jpg") || f.getName().toLowerCase().endsWith(".gif");
				}

				public String getDescription()
				{
					return "*.jpg;*.gif;";
				}
			} );
			d.showOpenDialog( null );
			File file = d.getSelectedFile();
			if (file != null) {
				mapPath = file.getPath();
				//System.out.println("map path set to: " + mapPath);
				mapString = "Map: " + file.getName();
				mapLabel.setForeground(new Color(50,150,50));
				mapLabel.setText(mapString);
				viewComponent.setImage( file );
				viewComponent.setDoubleBuffered(true);
				viewComponent.addMouseListener(viewComponent);
				viewComponent.addKeyListener(this);
			}
			pictureScrollPane.setViewportView(viewComponent);
		}
		// Load the height buffer
		else if ("buf".equals(e.getActionCommand())){
			JFileChooser d = new JFileChooser();
			d.setFileFilter( new FileFilter() {
				public boolean accept(File f)
				{
					return f.isDirectory() || f.getName().toLowerCase().endsWith(".buf");
				}

				public String getDescription()
				{
					return "*.buf";
				}
			} );
			d.showOpenDialog( null );
			final File file = d.getSelectedFile();
			if (file != null) {
				// TODO: Load Buffer
				bufPath = file.getPath();
				//System.out.println("buffer path set to: " + bufPath);
				bufString = "Loading Heightbuffer...";
				bufLabel.setForeground(new Color(50,50,150));
				bufLabel.setText(bufString);
				try {
					final RandomAccessFile raf = new RandomAccessFile(file,"r");
					if (buf == null) buf = new CGFloatBuffer(1024,1024);
					// THREAD
					new Thread() {
						public void run() {
							pb.setVisible(true);
							buf.fillBuffer(raf);
							bufString = "Heightbuffer: " + file.getName();
							bufLabel.setForeground(new Color(50,150,50));
							bufLabel.setText(bufString);
						}
					}.start();
				}
				catch (Exception E){
					System.out.println("File not found: " + file.getName());
				}		
			}
		}
		// Load the objects table
		if ("obj".equals(e.getActionCommand())){
			JFileChooser d = new JFileChooser();
			d.setFileFilter( new FileFilter() {
				public boolean accept(File f)
				{
					return f.isDirectory() || f.getName().toLowerCase().endsWith(".ob");
				}

				public String getDescription()
				{
					return "*.ob";
				}
			} );
			d.showOpenDialog( null );
			File file = d.getSelectedFile();
			if (file != null) {
				objPath = file.getPath();
				//System.out.println("object path set to: " + objPath);
				try {
					// TODO: Load Table of Objects	
					RandomAccessFile raf = new RandomAccessFile(file,"r");
					CGLoadFileParser.parseObjects(raf);
					raf.close();
					
					table = null;
					initialize();	// TODO: IST DAS OK SO?

					objString = "Objects Table: " + file.getName();
					objLabel.setForeground(new Color(50,150,50));
					objLabel.setText(objString);
				}
				catch (Exception E){
					System.out.println("File not found: " + file.getName());
					System.out.println("Error: " + E.getMessage());
				}					
			}
		}
		// Save the current project
		else if ("save".equals(e.getActionCommand())){
			JFileChooser d = new JFileChooser();
			d.setFileFilter( new FileFilter() {
				public boolean accept(File f)
				{
					return f.isDirectory() || f.getName().toLowerCase().endsWith(".cgl");
				}

				public String getDescription()
				{
					return "*.cgl";
				}
			} );
			d.showSaveDialog( null );
			File file = d.getSelectedFile();
			final JFrame fr = this;
			if (file != null) {
				final String filename = file.getPath()+".cgl";
				try{
		    		RandomAccessFile raf = new RandomAccessFile(filename, "rw");
		    		CGSaveWriter sw = new CGSaveWriter(raf);
		    		sw.save();
			        raf.close();
				}
				catch(Exception ex){
					System.out.println("Error: " + ex.getMessage());
				}
			}
		}
		// Load a project
		else if ("load".equals(e.getActionCommand())){
			JFileChooser d = new JFileChooser();
			d.setFileFilter( new FileFilter() {
				public boolean accept(File f)
				{
					return f.isDirectory() || f.getName().toLowerCase().endsWith(".cgl");
				}

				public String getDescription()
				{
					return "*.cgl";
				}
			} );
			d.showOpenDialog( null );
			File file = d.getSelectedFile();
			if (file != null) {
				try {
					RandomAccessFile raf = new RandomAccessFile(file,"r");
					CGLoadFileParser.parseSavedFile(raf);
					raf.close();
				}
				catch (Exception E){
					System.out.println("File not found: " + file.getName());
					System.out.println("Error: " + E.getMessage());
				}					
			}
		}
		// Create an XML File
		else if ("create".equals(e.getActionCommand())){
			JFileChooser d = new JFileChooser();
			d.setFileFilter( new FileFilter() {
				public boolean accept(File f)
				{
					return f.isDirectory() || f.getName().toLowerCase().endsWith(".xml");
				}

				public String getDescription()
				{
					return "*.xml";
				}
			} );
			d.showSaveDialog( null );
			File file = d.getSelectedFile();
			final JFrame fr = this;
			if (file != null) {
				final String filename = file.getPath()+".xml";
				try{
			    	if(everythingSet()){
						// THREAD
						new Thread() {
							public void run() {
								pb.setValue(0);
								pb.setMaximum(obj.getData().size());
								pb.setVisible(true);
								try{
						    		RandomAccessFile raf = new RandomAccessFile(filename, "rw");
						    		CGXMLWriter xw = new CGXMLWriter(raf);
						    		xw.writeXML();
							        raf.close();
									JOptionPane.showMessageDialog(fr,
										    "The XML file has been built.",
										    "Build successfully", 
										    JOptionPane.INFORMATION_MESSAGE);
								}
								catch(Exception ex){
									System.out.println("Error: " + ex.getMessage());
								}
								pb.setVisible(false);
							}
						}.start();
				    }
			   }
			   catch (Exception ex){
			        System.out.println("Fehler: " + ex);
			   }
			}
		}
		// Set multiplication factor for [0;1] normed positions and sizes
		else if ("multi".equals(e.getActionCommand())){
			setFrame.setVisible( true );
		}
		// Set header and footer
		else if ("hefo".equals(e.getActionCommand())){
			hefoFrame.setVisible( true );
		}
		// Add a new Object to the map
		else if ("addobject".equals(e.getActionCommand())){
			int id = obj.getSelectedRow();
			if (id!=-1){
				Vector temp = (Vector)(obj.getData().elementAt(id));
				CGObject ob = (CGObject)(temp.elementAt(3));
				CGAddedObject ao = new CGAddedObject(ob.getWidthX(),ob.getWidthZ(),ob.getName(),ob.getColor(),ob.getAttributes());
				viewComponent.addObject(ao);
			}
		}
	}

	public void keyPressed(KeyEvent e) {
		int kc = e.getKeyCode();
		// System.out.println("Key02: " + e.getKeyCode());
		if (viewComponent!=null){
			if (viewComponent.getCurrentObject() != null){
				// Objekt behandeln
				if (kc == 65)  viewComponent.getCurrentObject().moveLeft();
				if (kc == 83)  viewComponent.getCurrentObject().moveRight();
				if (kc == 87)  viewComponent.getCurrentObject().moveDown();
				if (kc == 89)  viewComponent.getCurrentObject().moveUp();
				if (kc == 77)  viewComponent.getCurrentObject().rotateY();
				if (kc == 78)  viewComponent.getCurrentObject().rotateYcc();
				if (kc == 74)  viewComponent.getCurrentObject().resetAngle();
				if (kc == 127) viewComponent.getObjectVector().remove(viewComponent.getCurrentObject());
				viewComponent.repaint();
			}
		}
	}

	public void keyTyped(KeyEvent e)    {	/* do nothing */	}

	public void keyReleased(KeyEvent e) {	/* do nothing */	}
}
