package util;

import java.io.RandomAccessFile;
import java.nio.ByteBuffer;

import gui.*;

public class CGFloatBuffer {
	private int width, height;
	private float[] data;
	
	// Constructor
	public CGFloatBuffer(int w, int h){
		width  = w;
		height = h;
		// initialize the buffer
		data = new float[width*height*4];
		for(int i=0; i < height; i++)
		{
			for(int j=0; j < width; j++)
			{
				int p = i * width + j;
				data[p * 4    ] = (float)0;
				data[p * 4 + 1] = (float)0;
				data[p * 4 + 2] = (float)0;
				data[p * 4 + 3] = (float)255;
			}
		}
	}
	
	// This Method is used to write the contents of a .buf File into the float buffer
	public void fillBuffer(RandomAccessFile raf){
		try{
			for (int i=0; i<(width*height*4); i++){
				byte[] b = new byte[4];
				raf.read(b);
				ByteBuffer bb = ByteBuffer.allocate(b.length);
				bb.put(b);
				bb.rewind();
				bb.order(java.nio.ByteOrder.LITTLE_ENDIAN);
				float f = bb.getFloat();
				data[i] = f;
				CGMainFrm.getProgressBar().setValue(i);
			}
			CGMainFrm.getProgressBar().setVisible(false);
		}
		catch (Exception e){
			System.out.println("Error while reading Buffer file: " + e.getMessage());
		}
	}
	
	// This method is used to get the red value at a specified pixel position
	public float getRed(int x, int y)
	{
		return data[(y*width + x)*4];
	}

	// This method is used to get the green value at a specified pixel position
	public float getGreen(int x, int y)
	{
		return data[(y*width + x)*4 + 1];
	}

	// This method is used to get the blue value at a specified pixel position
	public float getBlue(int x, int y)
	{
		return data[(y*width + x)*4 + 2];
	}

	// This method is used to get the alpha value at a specified pixel position
	public float getAlpha(int x, int y)
	{
		return data[(y*width + x)*4 + 3];
	}
}
