package util;

import java.awt.Color;
import java.io.*;
import gui.*;
import objects.*;

public class CGLoadFileParser {
	
	// Method to parse an object file (i.e. create object objects from it)
	public static void parseObjects(RandomAccessFile raf){
		String line = "";
		String[] tok;
		
		while (true){
			line = readLine(raf);
			if (line.equals("end")) break;	// Stop if the end of the file has been reached
			tok = getTokens(line);
			
			// We have a control point
			if (tok[0].equals("controlpoint")){
				float f1 = (new Float(tok[1])).floatValue();
				float f2 = (new Float(tok[2])).floatValue();
				String n = tok[0];
				Color c = getColor(tok[3]);
				CGObject ob = CGObjects.createObject(f1,f2,n,c,new String[1]);
				CGMainFrm.getObjects().addDataRow(ob);
				//System.out.println(CGMainFrm.getObjects().getData().size());
			}
			// Or a carstartposition
			else if (tok[0].equals("carstartposition")){
				float f1 = (new Float(tok[1])).floatValue();
				float f2 = (new Float(tok[2])).floatValue();
				String n = tok[0];
				Color c = getColor(tok[3]);
				CGObject ob = CGObjects.createObject(f1,f2,n,c,new String[1]);
				CGMainFrm.getObjects().addDataRow(ob);
				//System.out.println(CGMainFrm.getObjects().getData().size());
			}
			// Or a model
			else if (tok[0].equals("model")){
				float f1 = (new Float(tok[8])).floatValue();
				float f2 = (new Float(tok[9])).floatValue();
				String n = tok[0]+ " " + tok[2];
				Color c = getColor(tok[10]);
				String[] a = new String []{ tok[1],tok[3],tok[4],tok[5],tok[6],tok[7] };
				CGObject ob = CGObjects.createObject(f1,f2,n,c,a);
				CGMainFrm.getObjects().addDataRow(ob);
			}
		}
	}
	
	// Mathod to parse a saved .cgl File
	public static void parseSavedFile(RandomAccessFile raf){
		String line = "";
		String[] tok;
		
		while (true){
			line = readLine(raf);
			if (line.equals("end")) break;	// Stop if the end of the file has been reached
			tok = getTokens(line);
			
			// First treat the map, buffer and objects file paths
			if (tok[0].equals("map") && !tok[1].equals("")){
				CGMainFrm.setImageFromPath(tok[1]);
			}	
			if (tok[0].equals("buf") && !tok[1].equals("")){
				CGMainFrm.setBufferFromPath(tok[1]);
			}	
			if (tok[0].equals("obj") && !tok[1].equals("")){
				CGMainFrm.setObjectsFromPath(tok[1]);
			}	
			if (tok[0].equals("factor") && !tok[1].equals("")){
				CGMainFrm.setFactor((new Float(tok[1])).floatValue());
			}
			// now look at the objects
			if (tok[0].charAt(0) == '0') {
				String[] s = new String[6];
				for (int i=0;i<s.length;i++){
					s[i]=tok[i+8];
					//System.out.print(s[i]+"|");
				}
				//System.out.println();
				// CGAddedObject (float wX, float wZ, String n, Color c, Color cBU, float pX, float pZ, float an, String[] a){
				float wx = (new Float (tok[0])).floatValue();
				float wz = (new Float (tok[1])).floatValue();
				String n = tok[4];
				Color c = getColor(tok[6]);
				Color cBU = getColor(tok[7]);
				float pX = (new Float (tok[2])).floatValue()/CGMainFrm.getFactor();
				float pZ = (new Float (tok[3])).floatValue()/CGMainFrm.getFactor();
				float an = (new Float (tok[5])).floatValue();
				CGAddedObject ao = new CGAddedObject(wx,wz,n,c,cBU,pX,pZ,an,s);
				CGMainFrm.addObjectDirect(ao);
			}
		}
	}
	
	// Method to read one line of the objects file that is not empty or a comment
	private static String readLine(RandomAccessFile raf){
		String s = "%";
		try{
			while(s.length()==0 || s.charAt(0)=='%'){
				if(raf.getFilePointer()==raf.length()) return "end";
				s = raf.readLine();	
			}
		}
		catch (IOException e){
			System.out.println("Error: " + e.getMessage());
		}
		return s;
	}
	
	// Method to convert a line to an array of Strings that contains this line's tokens
	private static String[] getTokens(String str){
		String[] s = new String[20];
		String temp = "";
		int i = 0, k = 0;
		while(true){
			if (str.charAt(i)!=','&&str.charAt(i)!=';'){
				temp+=str.charAt(i);
			}
			if (str.charAt(i)==',' || str.charAt(i)==';'){
				// Debug Output
				// System.out.println("New Token: s[" + k + "]=" + temp);
				s[k]=temp;
				temp = "";
				k++;
			}
			if (str.charAt(i)==';') break;
			i++;
		}
		return s;
	}
	
	private static Color getColor(String s){
		int r = -1,g = -1,b = -1;
		String rs = "",gs = "",bs = "";
		for (int i=0;i<=s.length()-1;i++){
			if(s.charAt(i)!='-'){
				if(r==-1) rs+=s.charAt(i);
				else if((r!=-1)&&(g==-1)) gs+=s.charAt(i);
				else if((r!=-1)&&(g!=-1)&&(b==-1)) bs+=s.charAt(i);
			}
			if((s.charAt(i)=='-') && (r==-1))  r = (new Integer(rs)).intValue();
			else if((r!=-1) && (s.charAt(i)=='-') && (g==-1))  g = (new Integer(gs)).intValue();
		}
		b = (new Integer(bs)).intValue();
		// System.out.println(s + ": " + r + "/" + g + "/" + b);
		if (r>255) r = 255;
		if (g>255) g = 255;
		if (b>255) b = 255;
		if (r<0) r = 0;
		if (g<0) g = 0;
		if (b<0) b = 0;
		return (new Color(r,g,b));
	}
}
