package util;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

import objects.CGAddedObject;

import gui.*;

public class CGXMLWriter {
	private RandomAccessFile raf;
	
	public CGXMLWriter (RandomAccessFile r){
		raf = r;
	}
	
	public void writeXML(){
		try{
			Vector ob = CGMainFrm.getAddedObjects();
			
			// Write Header
			raf.writeBytes(CGMainFrm.getHeader());
			
			// Loop through controlpoints
			raf.writeBytes("\t<ControlPoints>\n");
			for (Enumeration en = ob.elements(); en.hasMoreElements(); ){
				CGAddedObject ao = ((CGAddedObject)(en.nextElement()));
				if(ao.getName().equals("controlpoint")){
					raf.writeBytes(/*"\t\t<Point x=\"" + ao.getPosition().x + 
							  		"\" z=\"" +  + ao.getPosition().y + 
							   		"\" angle=\"" + ao.getAngle() + 
							   		"\" rotX=\"0\" rotY=\"1\" rotZ=\"0\"" + " />\n");*/
									"\t\t<Point x1=\"" + (ao.getPosition().x-(multiplicate(ao.getWidthX()))/2.0) + 
								    "\" z1=\"" +  + (ao.getPosition().y-(multiplicate(ao.getWidthZ()))/2.0) + 
								    "\" x2=\"" + (ao.getPosition().x+(multiplicate(ao.getWidthX()))/2.0) +
								    "\" z2=\"" +  + (ao.getPosition().y+(multiplicate(ao.getWidthZ()))/2.0) + 
								    "\" angle=\"" + ao.getAngle() + 
								    "\" rotX=\"0\" rotY=\"1\" rotZ=\"0\"" + " />\n"); 
					CGMainFrm.getProgressBar().setValue(CGMainFrm.getProgressBar().getValue()+1);
				}
			}
			raf.writeBytes("\t</ControlPoints>\n");
			
			// Loop through car positions
			raf.writeBytes("\t<CarStartPositions>\n");
			for (Enumeration en = ob.elements(); en.hasMoreElements(); ){
				CGAddedObject ao = ((CGAddedObject)(en.nextElement()));
				if(ao.getName().equals("carstartposition")){
					raf.writeBytes("\t\t<Car x=\"" + /*(*/ao.getPosition().x/*-(multiplicate(ao.getWidthX()))/2.0)*/ +
								   "\" y=\"" + (CGMainFrm.getBuffer().getRed(ao.getPosition().x,ao.getPosition().y)) + 
								   "\" z=\"" + /*(*/ao.getPosition().y/*-(multiplicate(ao.getWidthZ()))/2.0)*/ +
								   "\" angle=\"" + ao.getAngle() + 
								   "\" rotX=\"0\" rotY=\"1\" rotZ=\"0\"" + " />\n");
					CGMainFrm.getProgressBar().setValue(CGMainFrm.getProgressBar().getValue()+1);
				}
			}
			raf.writeBytes("\t</CarStartPositions>\n");
			
			// Loop through models
			raf.writeBytes("\t<Models>\n");
			for (Enumeration en = ob.elements(); en.hasMoreElements(); ){
				CGAddedObject ao = ((CGAddedObject)(en.nextElement()));
				if(!ao.getName().equals("controlpoint") && !ao.getName().equals("carstartposition")){
					raf.writeBytes("\t\t<Model type=\"" + ao.getAttributes()[0] +
								   "\" name=\"" + ao.getName().substring(6) + 
								   "\" forceloading=\"" + ao.getAttributes()[1] + 
								   "\" location=\"" + ao.getAttributes()[2] + 
								   "\" shader=\"" + ao.getAttributes()[3] + 
								   "\" points=\"" + ao.getAttributes()[4] + 
								   "\" round=\"" + ao.getAttributes()[5] + "\" />\n");
					raf.writeBytes("\t\t\t<Location x=\"" + /*(*/ao.getPosition().x/*-(multiplicate(ao.getWidthX()))/2.0)*/ +
							   "\" y=\"" + (CGMainFrm.getBuffer().getRed(ao.getPosition().x,ao.getPosition().y)) + 
							   "\" z=\"" + /*(*/ao.getPosition().y/*-(multiplicate(ao.getWidthZ()))/2.0)*/ +
							   "\" angle=\"" + ao.getAngle() + 
							   "\" rotX=\"0\" rotY=\"1\" rotZ=\"0\"" + " />\n");
					raf.writeBytes("\t\t</Model>\n");
					CGMainFrm.getProgressBar().setValue(CGMainFrm.getProgressBar().getValue()+1);
				}
			}
			raf.writeBytes("\t</Models>\n");
			
			// Write Footer
			raf.writeBytes(CGMainFrm.getFooter());
			
			CGMainFrm.getProgressBar().setValue(CGMainFrm.getProgressBar().getMaximum());
		}
		catch(IOException ex){
			System.out.println("Error: " + ex.getMessage());
		}
	}
	
	// This multiplicates the [0;1] normed sizes with the user defines multiplication factor
	private float multiplicate(float f){
		return (f*CGMainFrm.getFactor());
	}
}
